// File generated by CPPExt (Value)
//
//                     Copyright (C) 1991 - 2000 by  
//                      Matra Datavision SA.  All rights reserved.
//  
//                     Copyright (C) 2001 - 2004 by
//                     Open CASCADE SA.  All rights reserved.
// 
// This file is part of the Open CASCADE Technology software.
//
// This software may be distributed and/or modified under the terms and
// conditions of the Open CASCADE Public License as defined by Open CASCADE SA
// and appearing in the file LICENSE included in the packaging of this file.
//  
// This software is distributed on an "AS IS" basis, without warranty of any
// kind, and Open CASCADE SA hereby disclaims all such warranties,
// including without limitation, any warranties of merchantability, fitness
// for a particular purpose or non-infringement. Please see the License for
// the specific terms and conditions governing rights and limitations under the
// License.

#ifndef _math_BracketedRoot_HeaderFile
#define _math_BracketedRoot_HeaderFile

#ifndef _Standard_Boolean_HeaderFile
#include <Standard_Boolean.hxx>
#endif
#ifndef _Standard_Real_HeaderFile
#include <Standard_Real.hxx>
#endif
#ifndef _Standard_Integer_HeaderFile
#include <Standard_Integer.hxx>
#endif
#ifndef _Standard_OStream_HeaderFile
#include <Standard_OStream.hxx>
#endif
class StdFail_NotDone;
class math_Function;


#ifndef _Standard_HeaderFile
#include <Standard.hxx>
#endif
#ifndef _Standard_Macro_HeaderFile
#include <Standard_Macro.hxx>
#endif

//! This class implements the Brent method to find the root of a function <br>
//! located within two bounds. No knowledge of the derivative is required. <br>
class math_BracketedRoot  {

public:

    void* operator new(size_t,void* anAddress) 
      {
        return anAddress;
      }
    void* operator new(size_t size) 
      { 
        return Standard::Allocate(size); 
      }
    void  operator delete(void *anAddress) 
      { 
        if (anAddress) Standard::Free((Standard_Address&)anAddress); 
      }
 // Methods PUBLIC
 // 


//! The Brent method is used to find the root of the function F between <br>
//! the bounds Bound1 and Bound2 on the function F. <br>
//! If F(Bound1)*F(Bound2) >0 the Brent method fails. <br>
//! The tolerance required for the root is given by Tolerance. <br>
//! The solution is found when : <br>
//!    abs(Xi - Xi-1) <= Tolerance; <br>
//! The maximum number of iterations allowed is given by NbIterations. <br>
Standard_EXPORT math_BracketedRoot(math_Function& F,const Standard_Real Bound1,const Standard_Real Bound2,const Standard_Real Tolerance,const Standard_Integer NbIterations = 100,const Standard_Real ZEPS = 1.0e-12);
//! Returns true if the computations are successful, otherwise returns false. <br>
  Standard_Boolean IsDone() const;
//! returns the value of the root. <br>
//! Exception NotDone is raised if the minimum was not found. <br>
  Standard_Real Root() const;
//! returns the value of the function at the root. <br>
//! Exception NotDone is raised if the minimum was not found. <br>
  Standard_Real Value() const;
//! returns the number of iterations really done during the <br>
//! computation of the Root. <br>
//! Exception NotDone is raised if the minimum was not found. <br>
  Standard_Integer NbIterations() const;

//! Prints on the stream o information on the current state <br>
//!          of the object. <br>
Standard_EXPORT   void Dump(Standard_OStream& o) const;





protected:

 // Methods PROTECTED
 // 


 // Fields PROTECTED
 //


private: 

 // Methods PRIVATE
 // 


 // Fields PRIVATE
 //
Standard_Boolean Done;
Standard_Real TheRoot;
Standard_Real TheError;
Standard_Integer NbIter;


};


#include <math_BracketedRoot.lxx>



// other Inline functions and methods (like "C++: function call" methods)
//


#endif
