// File		Xw_PixMap.cxx
// Created	14 October 1999
// Author	VKH
// Updated      GG 06/03/00 Xw_PixMap MUST creates a Pixmap and no more an Image !!!
//              GG IMP100701 Add the "depth" field and method
//              to the pixmap object.


//-Copyright	MatraDatavision 1999

//-Version

#define xTRACE 1

//
#include <errno.h>
#include <stdio.h>

#include <Xw_PixMap.ixx>

#include <Xw_Window.hxx>
#include <Xw_Extension.h>

#include <TCollection_AsciiString.hxx>

XW_STATUS Xw_save_xwd_image ( void*, void*, char* );
XW_STATUS Xw_save_bmp_image ( void*, void*, char* );
XW_STATUS Xw_save_gif_image ( void*, void*, char* );

Standard_Integer Xw_PixMap::PreferedDepth(
			const Handle(Aspect_Window)& aWindow,
		     	const Standard_Integer aDepth) const {
  Handle(Xw_Window) hwindow = Handle(Xw_Window)::DownCast(aWindow);
  XW_EXT_WINDOW *pwindow = (XW_EXT_WINDOW*) hwindow->ExtendedWindow();
  if( aDepth <= 0 ) return _DEPTH;
// Gets the nearest depth from possible screen depths
  Standard_Integer i,j;
  Screen *scr = DefaultScreenOfDisplay(_DISPLAY);
  for( i=j=0 ; i<scr->ndepths ; i++ ) {
    if( Abs(aDepth - scr->depths[i].depth) <
		Abs(aDepth - scr->depths[j].depth) ) {
      j = i;
    }
  }
  return scr->depths[j].depth;
}


//////////////////////////////////////////////////////////////////////////////////////////
Xw_PixMap::Xw_PixMap ( const Handle(Aspect_Window)& aWindow,
                       const Standard_Integer aWidth,
                       const Standard_Integer anHeight,
		       const Standard_Integer aDepth ) : 
	Aspect_PixMap(aWidth, anHeight, PreferedDepth(aWindow, aDepth))
{

  myWindow = Handle(Xw_Window)::DownCast(aWindow);

  XW_EXT_WINDOW *pwindow = (XW_EXT_WINDOW*) myWindow->ExtendedWindow();

  Xw_print_error();
  if( !Xw_get_trace() ) Xw_set_synchronize(_DISPLAY,True);
  myPixmap = XCreatePixmap( _DISPLAY, _WINDOW, myWidth, myHeight, myDepth); 
  if( !Xw_get_trace() ) Xw_set_synchronize(_DISPLAY,False);

  if ( !myPixmap || Xw_print_error() ) {
    char errstring[256];
    sprintf(errstring," *** Xw_PixMap : Cann't allocates pixmap of size %d x %d, request failed with errno : '%s'",myWidth, myHeight,strerror(errno));
    Aspect_PixmapDefinitionError::Raise ( errstring );
  }

}

///////////////////////////////
void 
Xw_PixMap::Destroy ()
{
  if( myPixmap ) {
#ifdef TRACE
    printf(" $$$ Xw_PixMap::Destroy() %x\n",myPixmap);
#endif
    XW_EXT_WINDOW *pwindow = (XW_EXT_WINDOW*) myWindow->ExtendedWindow();
    XFreePixmap(_DISPLAY,myPixmap);
  }
}


////////////////////////////////////////////////////////////
Standard_Boolean 
Xw_PixMap::Dump ( const Standard_CString aFilename,
		  const Standard_Real aGammaValue ) const
{
XW_STATUS result = XW_ERROR;
  
  if( myPixmap ) {
    XW_EXT_WINDOW *pwindow = (XW_EXT_WINDOW*) myWindow->ExtendedWindow();
    XW_EXT_IMAGEDATA *pimage = 
	(XW_EXT_IMAGEDATA*) Xw_get_image_from_pixmap( pwindow, NULL,
		myPixmap, myWidth/2, myHeight/2, myWidth, myHeight);
    if( pimage ) {
      if( aGammaValue != 1.0 ) 
	result = Xw_gamma_image( pimage, Standard_ShortReal(aGammaValue) );
      result = Xw_save_image( pwindow, pimage, (Standard_PCharacter)aFilename );

      Xw_close_image ( pimage );
    }
    if ( result == XW_ERROR ) Xw_print_error();
  } else 
    printf(" *** Xw_PixMap::Dump() Failed, the pixmap ID is NULL ***\n");

  return (result != XW_ERROR) ? Standard_True : Standard_False; 
}


////////////////////////////////////////////////////////////
Aspect_Handle Xw_PixMap::PixmapID() const
{
  return myPixmap;
}
